@@ -1,12 +1,16 @@
 package Geo::ShapeFile;
+
 use strict;
 use warnings;
 use Carp;
 use IO::File;
 use Geo::ShapeFile::Shape;
 use Config;
+use List::Util qw /min max/;
 
-our $VERSION = '2.52';
+our $VERSION = '2.54';
+
+my $is_little_endian = unpack 'b', (pack 'S', 1 );
 
 # Preloaded methods go here.
 sub new {
@@ -19,20 +23,23 @@ sub new {
     $self->{filebase} =~ s/\.\w{3}$//;
 
     $self->{_enable_caching} = {
-        shp                => 1,
-        dbf                => 1,
-        shx                => 1,
-        shapes_in_area    => 1,
+        shp            => 1,
+        dbf            => 1,
+        shx            => 1,
+        shapes_in_area => 1,
     };
+    $self->{has_shx} = 0;
+    $self->{has_shp} = 0;
+    $self->{has_dbf} = 0;
 
-    bless($self, $class);
+    bless $self, $class;
 
     $self->{_change_cache} = {
-        shape_type    => undef,
-        records        => undef,
-        shp    => {},
-        dbf    => {},
-        shx    => {},
+        shape_type => undef,
+        records    => undef,
+        shp        => {},
+        dbf        => {},
+        shx        => {},
     };
     $self->{_object_cache} = {
         shp    => {},
@@ -41,25 +48,19 @@ sub new {
         shapes_in_area => {},
     };
 
-    if(-f $self->{filebase}.".shx") {
+    if (-f $self->{filebase} . '.shx') {
         $self->read_shx_header();
         $self->{has_shx} = 1;
-    } else {
-        $self->{has_shx} = 0;
     }
 
-    if(-f $self->{filebase}.".shp") {
+    if (-f $self->{filebase} . '.shp') {
         $self->read_shp_header();
         $self->{has_shp} = 1;
-    } else {
-        $self->{has_shp} = 0;
     }
 
-    if(-f $self->{filebase}.".dbf") {
+    if (-f $self->{filebase} . '.dbf') {
         $self->read_dbf_header();
         $self->{has_dbf} = 1;
-    } else {
-        $self->{has_dbf} = 0;
     }
 
     return $self;
@@ -68,9 +69,10 @@ sub new {
 sub caching {
     my $self = shift;
     my $what = shift;
+    my $flag = shift;
 
-    if(@_) {
-        $self->{_enable_caching}->{$what} = shift;
+    if (defined $flag) {
+        $self->{_enable_caching}->{$what} = $flag;
     }
     return $self->{_enable_caching}->{$what};
 }
@@ -78,42 +80,48 @@ sub caching {
 sub cache {
     my $self = shift;
     my $type = shift;
-    my $obj = shift;
+    my $obj  = shift;
 
-    if($self->{_change_cache}->{$type} && $self->{_change_cache}->{$type}->{$obj}) {
-        return $self->{_change_cache}->{$type}->{$obj};
-    }
+    return $self->{_change_cache}->{$type}->{$obj}
+      if $self->{_change_cache}->{$type} && $self->{_change_cache}->{$type}->{$obj};
 
-    return unless $self->caching($type);
+    return if !$self->caching($type);
 
-    if($@) {
+    if (@_) {
         $self->{_object_cache}->{$type}->{$obj} = shift;
     }
     return $self->{_object_cache}->{$type}->{$obj};
 }
 
-sub read_shx_header { shift()->read_shx_shp_header('shx',@_); }
-sub read_shp_header { shift()->read_shx_shp_header('shp',@_); }
+
+sub read_shx_header {
+    shift()->read_shx_shp_header('shx', @_);
+}
+
+sub read_shp_header {
+    shift()->read_shx_shp_header('shp', @_);
+}
+
 sub read_shx_shp_header {
-    my $self = shift;
+    my $self  = shift;
     my $which = shift;
     my $doubles;
 
-    $self->{$which."_header"} = $self->get_bytes($which,0,100);
+    $self->{$which . '_header'} = $self->get_bytes($which, 0, 100);
     (
-        $self->{$which."_file_code"}, $self->{$which."_file_length"},
-        $self->{$which."_version"}, $self->{$which."_shape_type"}, $doubles
-    ) = unpack("N x20 N V2 a64",$self->{$which."_header"});
+        $self->{$which . '_file_code'}, $self->{$which . '_file_length'},
+        $self->{$which . '_version'},   $self->{$which . '_shape_type'}, $doubles
+    ) = unpack 'N x20 N V2 a64', $self->{$which . '_header'};
 
     (
-        $self->{$which."_x_min"}, $self->{$which."_y_min"},
-        $self->{$which."_x_max"}, $self->{$which."_y_max"},
-        $self->{$which."_z_min"}, $self->{$which."_z_max"},
-        $self->{$which."_m_min"}, $self->{$which."_m_max"},
+        $self->{$which . '_x_min'}, $self->{$which . '_y_min'},
+        $self->{$which . '_x_max'}, $self->{$which . '_y_max'},
+        $self->{$which . '_z_min'}, $self->{$which . '_z_max'},
+        $self->{$which . '_m_min'}, $self->{$which . '_m_max'},
     ) = (
-        unpack( 'b', pack( 'S', 1 ) )
-            ? unpack( 'd8', $doubles )
-            : reverse( unpack( 'd8', scalar( reverse( $doubles ) ) ) )
+        $is_little_endian
+            ? (unpack 'd8', $doubles )
+            : (reverse unpack 'd8', scalar reverse $doubles)
     );
 
     return 1;
@@ -123,13 +131,25 @@ sub type_is {
     my $self = shift;
     my $type = shift;
 
-    return(lc($self->type($self->shape_type)) eq lc($type));
+    return (lc $self->type($self->shape_type)) eq (lc $type);
+}
+
+sub get_dbf_field_names {
+    my $self = shift;
+
+    croak 'dbf field names not loaded yet'
+      if !defined $self->{dbf_field_names};
+
+    #  make sure we return a copy
+    my @fld_names = @{$self->{dbf_field_names}};
+
+    return wantarray ? @fld_names : \@fld_names;
 }
 
 sub read_dbf_header {
     my $self = shift;
 
-    $self->{dbf_header} = $self->get_bytes('dbf',0,12);
+    $self->{dbf_header} = $self->get_bytes('dbf', 0, 12);
     (
         $self->{dbf_version},
         $self->{dbf_updated_year},
@@ -138,35 +158,34 @@ sub read_dbf_header {
         $self->{dbf_num_records},
         $self->{dbf_header_length},
         $self->{dbf_record_length},
-    ) = unpack("C4 V v v", $self->{dbf_header});
+    ) = unpack 'C4 V v v', $self->{dbf_header};
     # unpack changed from c4 l s s to fix endianess problem
     # reported by Daniel Gildea
 
-    my $ls = $self->{dbf_header_length} +
-        ($self->{dbf_num_records}*$self->{dbf_record_length});
-    my $li = -s $self->{filebase}.".dbf";
+    my $ls = $self->{dbf_header_length}
+           + $self->{dbf_num_records} * $self->{dbf_record_length};
+    my $li = -s $self->{filebase} . '.dbf';
 
     # some shapefiles (such as are produced by the NOAA NESDIS) don't
     # have a end-of-file marker in their dbf files, Aleksandar Jelenak
     # says the ESRI tools don't have a problem with this, so we shouldn't
     # either
-    my $last_byte = $self->get_bytes('dbf',$li-1,1);
-    $ls += 1 if (ord $last_byte == 0x1A);
+    my $last_byte = $self->get_bytes('dbf', $li-1, 1);
+    $ls ++ if ord $last_byte == 0x1A;
 
-    if($ls != $li) {
-        croak "dbf: file wrong size (should be $ls, but found $li)";
-    }
+    croak "dbf: file wrong size (should be $ls, but found $li)"
+      if $ls != $li;
 
-    my $header = $self->get_bytes('dbf',32,$self->{dbf_header_length}-32);
+    my $header = $self->get_bytes('dbf', 32, $self->{dbf_header_length} - 32);
     my $count = 0;
     $self->{dbf_header_info} = [];
 
-    while($header) {
-        my $tmp = substr($header,0,32,'');
-        my $chr = substr($tmp,0,1);
+    while ($header) {
+        my $tmp = substr $header, 0, 32, '';
+        my $chr = substr $tmp, 0, 1;
 
-        if(ord $chr == 0x0D) { last; }
-        if(length($tmp) < 32) { last; }
+        last if ord $chr == 0x0D;
+        last if length ($tmp) < 32;
 
         my %tmp = ();
         (
@@ -174,34 +193,36 @@ sub read_dbf_header {
             $tmp{type},
             $tmp{size},
             $tmp{decimals}
-        ) = unpack("Z11 Z x4 C2",$tmp);
+        ) = unpack 'Z11 Z x4 C2', $tmp;
 
         $self->{dbf_field_info}->[$count] = {%tmp};
-        
+
         $count++;
     }
+
     $self->{dbf_fields} = $count;
-    if($count < 1) { croak "dbf: Not enough fields ($count < 1)"; }
+    croak "dbf: Not enough fields ($count < 1)"
+      if $count < 1;
 
     my @template = ();
-    foreach(@{$self->{dbf_field_info}}) {
-        if($_->{size} < 1) {
-            croak "dbf: Field $_->{name} too short ($_->{size} bytes)";
-        }
-        if($_->{size} > 4000) {
-            croak "dbf: Field $_->{name} too long ($_->{size} bytes)";
-        }
+    foreach (@{$self->{dbf_field_info}}) {
+        croak "dbf: Field $_->{name} too short ($_->{size} bytes)"
+          if $_->{size} < 1;
 
-        push(@template,"A".$_->{size});
+        croak "dbf: Field $_->{name} too long ($_->{size} bytes)"
+          if $_->{size} > 4000;
+
+        push @template, 'A' . $_->{size};
     }
-    $self->{dbf_record_template} = join(' ',@template);
+    $self->{dbf_record_template} = join ' ', @template;
 
     my @field_names = ();
-    foreach(@{$self->{dbf_field_info}}) {
-        push(@field_names,$_->{name});
+    foreach (@{$self->{dbf_field_info}}) {
+        push @field_names, $_->{name};
     }
     $self->{dbf_field_names} = [@field_names];
 
+    #  should return field names?  
     return 1;
 }
 
@@ -217,9 +238,11 @@ sub generate_dbf_header {
         $self->{dbf_num_records},
         $self->{dbf_header_length},
         $self->{dbf_record_length},
-    ) = unpack("C4 V v v", $self->{dbf_header});
+    ) = unpack 'C4 V v v', $self->{dbf_header};
 
-    $self->{_change_cache}->{dbf_cache}->{header} = pack("C4 V v v",
+    $self->{_change_cache}->{dbf_cache}->{header}
+      = pack
+        'C4 V v v',
         3,
         (localtime)[5],
         (localtime)[4]+1,
@@ -227,8 +250,9 @@ sub generate_dbf_header {
         0, # TODO - num_records,
         0, # TODO - header_length,
         0, # TODO - record_length,
-    );
+    ;
 
+#  SWL, 2014-02-09:  Unsure of the purpose of this commented code as yet
 #    my $ls = $self->{dbf_header_length} +
 #        ($self->{dbf_num_records}*$self->{dbf_record_length}) +
 #        1;
@@ -287,11 +311,12 @@ sub generate_dbf_header {
 }
 
 sub get_dbf_record {
-    my $self = shift;
+    my $self  = shift;
     my $entry = shift;
 
-    my $dbf = $self->cache('dbf',$entry);
-    if(! $dbf) {
+    my $dbf = $self->cache('dbf', $entry);
+
+    if (!$dbf) {
         $entry--; # make entry 0-indexed
 
         my $record = $self->get_bytes(
@@ -299,7 +324,7 @@ sub get_dbf_record {
             $self->{dbf_header_length}+($self->{dbf_record_length} * $entry),
             $self->{dbf_record_length}+1, # +1 for deleted flag
         );
-        my($del,@data) = unpack("c".$self->{dbf_record_template},$record);
+        my ($del, @data) = unpack 'c' . $self->{dbf_record_template}, $record;
 
         map { s/^\s*//; s/\s*$//; } @data;
 
@@ -307,19 +332,15 @@ sub get_dbf_record {
         @record{@{$self->{dbf_field_names}}} = @data;
         $record{_deleted} = (ord $del == 0x2A);
         $dbf = {%record};
-        $self->cache('dbf',$entry+1,$dbf);
+        $self->cache('dbf', $entry + 1, $dbf);
     }
 
-    if(wantarray) {
-        return %{$dbf};
-    } else {
-        return $dbf;
-    }
+    return wantarray ? %{$dbf} : $dbf;
 }
 
 sub set_dbf_record {
-    my $self = shift;
-    my $entry = shift;
+    my $self   = shift;
+    my $entry  = shift;
     my %record = @_;
 
     $self->{_change_cache}->{dbf}->{$entry} = {%record};
@@ -327,15 +348,16 @@ sub set_dbf_record {
 
 sub get_shp_shx_header_value {
     my $self = shift;
-    my $val = shift;
+    my $val  = shift;
 
-    unless($self->{"shx_".$val} || $self->{"shp_".$val}) {
+    if (!($self->{'shx_' . $val} || $self->{'shp_' . $val})) {
         $self->read_shx_header();
     }
 
-    return $self->{"shx_".$val} || $self->{"shp_".$val} || undef;
+    return $self->{'shx_' . $val} || $self->{'shp_' . $val} || undef;
 }
 
+#  factory these
 sub x_min { shift()->get_shp_shx_header_value('x_min'); }
 sub x_max { shift()->get_shp_shx_header_value('x_max'); }
 sub y_min { shift()->get_shp_shx_header_value('y_min'); }
@@ -348,39 +370,47 @@ sub m_max { shift()->get_shp_shx_header_value('m_max'); }
 sub upper_left_corner {
     my $self = shift;
 
-    return new Geo::ShapeFile::Point(X => $self->x_min, Y => $self->y_min);
+    return Geo::ShapeFile::Point->new(X => $self->x_min, Y => $self->y_max);
 }
+
 sub upper_right_corner {
     my $self = shift;
 
-    return new Geo::ShapeFile::Point(X => $self->x_max, Y => $self->y_min);
+    return Geo::ShapeFile::Point->new(X => $self->x_max, Y => $self->y_max);
 }
+
 sub lower_right_corner {
     my $self = shift;
 
-    return new Geo::ShapeFile::Point(X => $self->x_max, Y => $self->y_max);
+    return Geo::ShapeFile::Point->new(X => $self->x_max, Y => $self->y_min);
 }
+
 sub lower_left_corner {
     my $self = shift;
 
-    return new Geo::ShapeFile::Point(X => $self->x_min, Y => $self->y_max);
+    return Geo::ShapeFile::Point->new(X => $self->x_min, Y => $self->y_min);
 }
 
 sub height {
     my $self = shift;
 
-    return $self->x_max - $self->x_min;
+    return if !$self->records;
+
+    return $self->y_max - $self->y_min;
 }
+
 sub width {
     my $self = shift;
 
-    return $self->y_max - $self->y_min;
+    return if !$self->records;
+
+    return $self->x_max - $self->x_min;
 }
 
 sub corners {
     my $self = shift;
 
-    return(
+    return (
         $self->upper_left_corner,
         $self->upper_right_corner,
         $self->lower_right_corner,
@@ -389,24 +419,28 @@ sub corners {
 }
 
 sub area_contains_point {
-    my $self = shift;
+    my $self  = shift;
     my $point = shift;
-    my ($x_min,$y_min,$x_max,$y_max) = @_;
 
-    return (
+    my ($x_min, $y_min, $x_max, $y_max) = @_;
+
+    my $result =
         ($point->X >= $x_min) &&
         ($point->X <= $x_max) &&
         ($point->Y >= $y_min) &&
-        ($point->Y <= $y_max)
-    );
+        ($point->Y <= $y_max);
+
+    return $result;
 }
 
 sub bounds_contains_point {
-    my $self = shift;
+    my $self  = shift;
     my $point = shift;
 
-    return $self->area_contains_point(
-        $point, $self->x_min, $self->y_min, $self->x_max, $self->y_max,
+    return $self->area_contains_point (
+        $point,
+        $self->x_min, $self->y_min,
+        $self->x_max, $self->y_max,
     );
 }
 
@@ -417,40 +451,45 @@ sub file_version {
 sub shape_type {
     my $self = shift;
 
-    if(defined $self->{_change_cache}->{shape_type}) {
-        return $self->{_change_cache}->{shape_type};
-    } else {
-        return $self->get_shp_shx_header_value('shape_type');
-    }
+    return $self->{_change_cache}->{shape_type}
+      if defined $self->{_change_cache}->{shape_type};
+
+    return $self->get_shp_shx_header_value('shape_type');
 }
 
 sub shapes {
     my $self = shift;
 
-    if(defined $self->{_change_cache}->{records}) {
-        return $self->{_change_cache}->{records};
+    return $self->{_change_cache}->{records}
+      if defined $self->{_change_cache}->{records};
+
+    if (!$self->{shx_file_length}) {
+        $self->read_shx_header();
     }
-    unless($self->{shx_file_length}) { $self->read_shx_header(); }
 
     my $filelength = $self->{shx_file_length};
-    $filelength -= 50; # don't count the header
-    return ($filelength/4);
+    $filelength   -= 50; # don't count the header
+
+    return $filelength / 4;
 }
 
 sub records {
     my $self = shift;
 
-    if(defined $self->{_change_cache}->{records}) {
-        return $self->{_change_cache}->{records};
-    }
+    return $self->{_change_cache}->{records}
+      if defined $self->{_change_cache}->{records};
 
-    if($self->{shx_file_length}) {
+    if ($self->{shx_file_length}) {
         my $filelength = $self->{shx_file_length};
-        $filelength -= 50; # don't count the header
-        return ($filelength/4);
-    } elsif($self->{dbf_num_records}) {
+        $filelength   -= 50; # don't count the header
+        return $filelength / 4;
+    }
+    #  should perhaps just return dbf_num_records if we get this far?  
+    elsif ($self->{dbf_num_records}) {
         return $self->{dbf_num_records};
     }
+
+    return 0;
 }
 
 sub shape_type_text {
@@ -459,20 +498,26 @@ sub shape_type_text {
     return $self->type($self->shape_type());
 }
 
-sub get_shx_record_header { shift()->get_shx_record(@_); }
+sub get_shx_record_header {
+    shift()->get_shx_record(@_);
+}
+
 sub get_shx_record {
-    my $self = shift;
+    my $self  = shift;
     my $entry = shift;
 
-    croak "must specify entry index" unless $entry;
+    croak 'must specify entry index'
+      if !$entry;
 
-    my $shx = $self->cache('shx',$entry);
-    unless($shx) {
-        my $record = $self->get_bytes('shx',(($entry - 1) * 8) + 100,8);
-        $shx = [unpack("N N",$record)];
-        $self->cache('shx',$entry,$shx);
+    my $shx = $self->cache('shx', $entry);
+
+    if (!$shx) {
+        my $record = $self->get_bytes('shx', (($entry - 1) * 8) + 100, 8);
+        $shx = [unpack 'N N', $record];
+        $self->cache('shx', $entry,$shx);
     }
-    return(@{$shx});
+
+    return @{$shx};
 }
 
 sub get_shp_record_header {
@@ -482,9 +527,9 @@ sub get_shp_record_header {
     my($offset) = $self->get_shx_record($entry);
 
     my $record = $self->get_bytes('shp',$offset*2,8);
-    my($number,$content_length) = unpack("N N",$record);
+    my ($number, $content_length) = unpack 'N N', $record;
 
-    return($number,$content_length);
+    return ($number,$content_length);
 }
 
 # TODO - cache this
@@ -493,72 +538,81 @@ sub shapes_in_area {
     my @area = @_; # x_min,y_min,x_max,y_max,
 
     my @results = ();
-    for(1 .. $self->shapes) {
-        my($offset,$content_length) = $self->get_shx_record($_);
-        my $type = unpack("V",$self->get_bytes('shp',($offset*2)+8,4));
+    SHAPE:
+    for (1 .. $self->shapes) {
+        my($offset, $content_length) = $self->get_shx_record($_);
+        my $type = unpack 'V', $self->get_bytes('shp',$offset * 2 + 8, 4);
 
-        if($self->type($type) eq 'Null') {
-            next;
-        } elsif($self->type($type) =~ /^Point/) {
-            my $bytes = $self->get_bytes('shp',($offset*2)+12,16);
-            my($x,$y) = (
-                unpack( 'b', pack( 'S', 1 ) )
-                    ? unpack( 'dd', $bytes )
-                    : reverse( unpack( 'dd', scalar( reverse( $bytes ) ) ) )
+        next SHAPE if $self->type($type) eq 'Null';
+
+        if ($self->type($type) =~ /^Point/) {
+            my $bytes = $self->get_bytes('shp', $offset * 2 +12, 16);
+            my ($x, $y) = (
+                $is_little_endian
+                    ? (unpack 'dd', $bytes )
+                    : (reverse unpack 'dd', scalar reverse $bytes)
             );
-            my $pt = new Geo::ShapeFile::Point(X => $x, Y => $y);
+            my $pt = Geo::ShapeFile::Point->new(X => $x, Y => $y);
             if($self->area_contains_point($pt,@area)) {
-                push(@results,$_);
+                push @results, $_;
             }
-        } elsif($self->type($type) =~ /^(PolyLine|Polygon|MultiPoint|MultiPatch)/) {
+        }
+        elsif ($self->type($type) =~ /^(PolyLine|Polygon|MultiPoint|MultiPatch)/) {
             my $bytes = $self->get_bytes('shp',($offset*2)+12,32);
             my @p = (
-                unpack( 'b', pack( 'S', 1 ) )
-                    ? unpack( 'd4', $bytes )
-                    : reverse( unpack( 'd4', scalar( reverse( $bytes ) ) ) )
+                $is_little_endian
+                    ? (unpack 'd4', $bytes )
+                    : (reverse unpack 'd4', scalar reverse $bytes )
             );
-            if($self->check_in_area(@p,@area) || $self->check_in_area(@area,@p)) {
-                push(@results,$_);
+            if ($self->check_in_area(@p, @area)) {
+                push @results, $_;
             }
-        } else {
-            print "type=".$self->type($type)."\n";
+        }
+        else {
+            print 'type=' . $self->type($type) . "\n";
         }
     }
+
     return @results;
 }
 
 sub check_in_area {
     my $self = shift;
-    my(
-        $x1_min,$y1_min,$x1_max,$y1_max,
-        $x2_min,$y2_min,$x2_max,$y2_max,
+    my (
+        $x1_min, $y1_min, $x1_max, $y1_max,
+        $x2_min, $y2_min, $x2_max, $y2_max,
     ) = @_;
 
-    my $lhit = $self->between($x1_min,$x2_min,$x2_max);
-    my $rhit = $self->between($x1_max,$x2_min,$x2_max);
-    my $thit = $self->between($y1_min,$y2_min,$y2_max);
-    my $bhit = $self->between($y1_max,$y2_min,$y2_max);
-
-    return ( # collision
-        ($lhit && $thit) || ($rhit && $thit) || ($lhit && $bhit) || ($rhit && $bhit)
-    ) || ( # containment
-        ($lhit && $thit) && ($rhit && $thit) && ($lhit && $bhit) && ($rhit && $bhit)
+    my $result = !(
+           $x1_min > $x2_max
+        or $x1_max < $x2_min
+        or $y1_min > $y2_max
+        or $y1_max < $y2_min
     );
+
+    return $result;
 }
 
+#  SWL: not used anymore - remove?
 sub between {
-    my $self = shift;
-
+    my $self  = shift;
     my $check = shift;
 
-    unless($_[0] < $_[1]) { @_ = reverse @_; }
-    return (($check >= $_[0]) && ($check <= $_[1]));
+    #  ensure min then max
+    if ($_[0] > $_[1]) {
+        @_ = reverse @_;
+    }
+
+    return ($check >= $_[0]) && ($check <= $_[1]);
 }
 
 sub bounds {
     my $self = shift;
 
-    return($self->x_min,$self->y_min,$self->x_max,$self->y_max);
+    return (
+        $self->x_min, $self->y_min,
+        $self->x_max, $self->y_max,
+    );
 }
 
 sub extract_ints {
@@ -566,77 +620,90 @@ sub extract_ints {
     my $end = shift;
     my @what = @_;
 
-    my $template = ($end =~ /^l/i)?'V':'N';
+    my $template = ($end =~ /^l/i) ? 'V': 'N';
 
     $self->extract_and_unpack(4, $template, @what);
-    foreach(@what) {
+    foreach (@what) {
         $self->{$_} = $self->{$_};
     }
 }
 
 sub get_shp_record {
-    my $self = shift;
+    my $self  = shift;
     my $entry = shift;
 
     my $shape = $self->cache('shp',$entry);
-    unless($shape) {
+    if (!$shape) {
         my($offset,$content_length) = $self->get_shx_record($entry);
 
-        my $record = $self->get_bytes('shp',$offset*2,($content_length*2)+8);
+        my $record = $self->get_bytes('shp', $offset * 2, $content_length * 2 + 8);
 
-        $shape = new Geo::ShapeFile::Shape();
+        $shape = Geo::ShapeFile::Shape->new();
         $shape->parse_shp($record);
-        $self->cache('shp',$entry,$shape);
+        $self->cache('shp', $entry, $shape);
     }
 
     return $shape;
 }
 
-sub shx_handle { shift()->get_handle('shx'); }
-sub shp_handle { shift()->get_handle('shp'); }
-sub dbf_handle { shift()->get_handle('dbf'); }
+sub shx_handle {
+    shift()->get_handle('shx');
+}
+
+sub shp_handle {
+    shift()->get_handle('shp');
+}
+
+sub dbf_handle {
+    shift()->get_handle('dbf');
+}
+
 sub get_handle {
-    my $self = shift;
+    my $self  = shift;
     my $which = shift;
 
-    my $han = $which."_handle";
-    unless($self->{$han}) {
-        $self->{$han} = new IO::File;
-        my $file = join('.', $self->{filebase},$which);
-        unless($self->{$han}->open($file, O_RDONLY | O_BINARY)) {
-            croak "Couldn't get file handle for $file: $!";
-        }
-        binmode($self->{$han}); # fix windows bug reported by Patrick Dughi
+    my $han = $which . '_handle';
+
+    if (!$self->{$han}) {
+        $self->{$han} = IO::File->new;
+        my $file = join '.', $self->{filebase}, $which;
+        croak "Couldn't get file handle for $file: $!"
+          if not $self->{$han}->open($file, O_RDONLY | O_BINARY);
+        binmode $self->{$han}; # fix windows bug reported by Patrick Dughi
     }
 
     return $self->{$han};
 }
 
 sub get_bytes {
-    my $self = shift;
-    my $file = shift;
+    my $self   = shift;
+    my $file   = shift;
     my $offset = shift;
     my $length = shift;
 
-    my $handle = $file."_handle";
+    my $handle = $file . '_handle';
     my $h = $self->$handle();
-    $h->seek($offset,0) || confess "Couldn't seek to $offset for $file";;
+    $h->seek ($offset, 0)
+      || croak "Couldn't seek to $offset for $file";
+
     my $tmp;
-    my $res = $h->read($tmp,$length);
-    if(defined $res) {
-        if($res == 0) {
-            confess "EOF reading $length bytes from $file at offset $offset";
-        }
-    } else {
-        confess "Couldn't read $length bytes from $file at offset $offset ($!)";
-    }
+    my $res = $h->read($tmp, $length);
+
+    croak "Couldn't read $length bytes from $file at offset $offset ($!)"
+      if !defined $res;
+
+    croak "EOF reading $length bytes from $file at offset $offset"
+      if $res == 0;
+
     return $tmp;
 }
 
+
 sub type {
-    my $self = shift;
+    my $self  = shift;
     my $shape = shift;
 
+    #  should make this a package lexical
     my %shape_types = qw(
         0   Null
         1   Point
@@ -658,33 +725,45 @@ sub type {
 }
 
 sub find_bounds {
-    my $self = shift;
+    my $self    = shift;
     my @objects = @_;
 
-    my %bounds = (
-        x_min    => undef,
-        y_min    => undef,
-        x_max    => undef,
-        y_max    => undef,
-    );
+    return if !scalar @objects;
+
+    my $obj1 = shift @objects;
+
+    #  assign values from first object to start
+    my $x_min = $obj1->x_min();
+    my $y_min = $obj1->y_min();
+    my $x_max = $obj1->x_max();
+    my $y_max = $obj1->y_max();
+
 
     foreach my $obj (@objects) {
-        foreach('x_min','y_min') {
-            if((!defined $bounds{$_}) || ($obj->$_() < $bounds{$_})) {
-                $bounds{$_} = $obj->$_();
-            }
-        }
-        foreach('x_max','y_max') {
-            if((!defined $bounds{$_}) || ($obj->$_() > $bounds{$_})) {
-                $bounds{$_} = $obj->$_();
-            }
-        }
+        $x_min = min ($x_min, $obj->x_min());
+        $y_min = min ($y_min, $obj->y_min());
+        $x_max = max ($x_max, $obj->x_max());
+        $y_max = max ($y_max, $obj->y_max());
     }
-    return(%bounds);
+
+    my %bounds = (
+        x_min => $x_min,
+        y_min => $y_min,
+        x_max => $x_max,
+        y_max => $y_max,
+    );
+
+    return %bounds;
 }
 
+# XML::Generator::SVG::ShapeFile fails because it is calling this method
+# and it does not exist in 2.52 and earlier
+sub DESTROY {}
+
+
 1;
 __END__
+
 =head1 NAME
 
 Geo::ShapeFile - Perl extension for handling ESRI GIS Shapefiles.
@@ -693,7 +772,7 @@ Geo::ShapeFile - Perl extension for handling ESRI GIS Shapefiles.
 
   use Geo::ShapeFile;
 
-  my $shapefile = new Geo::ShapeFile("roads");
+  my $shapefile = Geo::ShapeFile->new('roads');
 
   for(1 .. $shapefile->shapes()) {
     my $shape = $shapefile->get_shp_record($_);
@@ -723,7 +802,7 @@ base) formats.
 Creates a new shapefile object, the only argument it takes is the basename
 for your data (don't include the extension, the module will automatically
 find the extensions it supports).  For example if you have data files called
-roads.shp, roads.shx, and roads.dbf, use 'new Geo::ShapeFile("roads");' to
+roads.shp, roads.shx, and roads.dbf, use 'Geo::ShapeFile->new("roads");' to
 create a new object, and the module will load the data it needs from the
 files as it needs it.
 
@@ -807,6 +886,7 @@ Returns the shape type of the current shp/shx file (see shape_type()), but
 as the human-readable string type, rather than an integer.
 
 =item get_shx_record($record_index)
+
 =item get_shx_record_header($record_index)
 
 Get the contents of an shx record or record header (for compatibility with
@@ -825,18 +905,12 @@ array consisting of the record number and the content length of the record.
 Retrieve an shp record for the specified index.  Returns a
 Geo::ShapeFile::Shape object.
 
-=item shapes_in_area($x_min,$y_min,$x_max,$y_max)
+=item shapes_in_area($x_min, $y_min, $x_max, $y_max)
 
-Returns an array of integers, consisting of the indices of the shapes that
-overlap with the area specified.  Currently this is a very oversimplified
-function that actually finds shapes that have any point that falls within
-the specified bounding box.  Currently it may miss some shapes that actually
-do overlap with the specified area, if there are two points outside the area
-that cause an edge to pass through the area, but neither of the end points
-of that edge actually fall within the area specified.  Patches to make this
-function more useful would be welcome.
+Returns an array of integers listing which shape IDs have
+bounding boxes that overlap with the area specified.
 
-=item check_in_area($x1_min,$y1_min,$x1_max,$y1_max,$x2_min,$x2_max,$y2_min,$y2_max)
+=item check_in_area($x1_min, $y1_min, $x1_max, $y1_max, $x2_min, $x2_max, $y2_min, $y2_max)
 
 Returns true if the two specified areas overlap.
 
@@ -856,27 +930,40 @@ Returns the name of the type associated with the given type id number.
 =item find_bounds(@shapes)
 
 Takes an array of Geo::ShapeFile::Shape objects, and returns a hash, with
-keys of x_min,y_min,x_max,y_max, with the values for each of those ranges.
+keys of x_min,y_min,x_max,y_max, with the values for each of those bounds.
+
+=item get_dbf_field_names()
+
+Returns an array of the field names in the dbf file, in file order.
+Returns an array reference if used in scalar context.  
 
 =back
 
 =head1 REPORTING BUGS
 
 Please send any bugs, suggestions, or feature requests to
-  E<lt>geo-shapefile-bugs@jasonkohles.comE<gt>.
+  L<https://github.com/shawnlaffan/Geo-ShapeFile/issues>.
 
 =head1 SEE ALSO
 
-Geo::ShapeFile::Shape
-Geo::ShapeFile::Point
+L<Geo::ShapeFile::Shape>, 
+L<Geo::ShapeFile::Point>, 
+L<Geo::Shapefile::Writer>, 
+L<Geo::GDAL>
 
 =head1 AUTHOR
 
 Jason Kohles, E<lt>email@jasonkohles.comE<gt>
 
+Shawn Laffan, E<lt>shawnlaffan@gmail.comE<gt>
+
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2002,2003 by Jason Kohles
+Copyright 2002-2013 by Jason Kohles (versions up to and including 2.52)
+
+Copyright 2014 by Shawn Laffan (versions 2.53 -)
+
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
